@@ -20,7 +20,7 @@
 
 using namespace Gudhi;
 
-struct MyOptions : Simplex_tree_options_full_featured {
+struct MyOptions : Simplex_tree_options_minimal {
   // Not doing persistence, so we don't need those
   static const bool store_key = false;
   static const bool store_filtration = false;
@@ -435,3 +435,136 @@ BOOST_AUTO_TEST_CASE(mini_prune_above_filtration) {
   std::clog << "The complex contains " << st.num_simplices() << " simplices" << std::endl;
 
 }
+
+BOOST_AUTO_TEST_CASE(prune_above_filtration_limits) {
+  std::clog << "********************************************************************" << std::endl;
+  std::clog << "PRUNE ABOVE FILTRATION TEST TO THE LIMITS" << std::endl;
+
+  Stree st;
+
+  st.insert_simplex_and_subfaces({0, 1, 6, 7}, 1.0);
+  st.insert_simplex_and_subfaces({3, 4, 5}, 2.0);
+
+  // As NaN is not yet managed in prune_above_filtration, this simplex would be removed if the Simplex tree is browsed
+  // when prune_above_filtration(+Inf)
+  st.insert_simplex_and_subfaces({3, 0}, std::numeric_limits<Stree::Filtration_value>::quiet_NaN());
+  st.insert_simplex_and_subfaces({2, 1, 0}, 4.0);
+
+  // st:
+  //    1   6
+  //    o---o
+  //   /X\7/
+  //  o---o---o---o
+  //  2   0   3\X/4
+  //            o
+  //            5
+  st.initialize_filtration();
+
+  std::clog << "The complex contains " << st.num_simplices() << " simplices" << std::endl;
+  std::clog << " - dimension " << st.dimension() << std::endl;
+  std::clog << "Iterator on Simplices in the filtration, with [filtration value]:" << std::endl;
+  for (auto f_simplex : st.filtration_simplex_range()) {
+    std::clog << "   " << "[" << st.filtration(f_simplex) << "] ";
+    for (auto vertex : st.simplex_vertex_range(f_simplex)) {
+      std::clog << (int) vertex << " ";
+    }
+    std::clog << std::endl;
+  }
+  BOOST_CHECK(st.num_simplices() == 27);
+
+  // Test case to the limit
+  bool simplex_is_changed = st.prune_above_filtration(std::numeric_limits<Stree::Filtration_value>::infinity());
+  BOOST_CHECK(simplex_is_changed == false);
+
+  // Here we cannot test with a copy, because NaN != NaN
+
+  std::clog << "The complex contains " << st.num_simplices() << " simplices" << std::endl;
+  BOOST_CHECK(st.num_simplices() == 27);
+}
+
+BOOST_AUTO_TEST_CASE(mini_prune_above_dimension) {
+  std::clog << "********************************************************************" << std::endl;
+  std::clog << "MINI PRUNE ABOVE DIMENSION" << std::endl;
+
+  Mini_stree st;
+
+  st.insert_simplex_and_subfaces({0, 1, 6, 7});
+  st.insert_simplex_and_subfaces({3, 4, 5});
+  st.insert_simplex_and_subfaces({3, 0});
+  st.insert_simplex_and_subfaces({2, 1, 0});
+
+  // st:
+  //    1   6
+  //    o---o
+  //   /X\7/
+  //  o---o---o---o
+  //  2   0   3\X/4
+  //            o
+  //            5
+
+  st.initialize_filtration();
+  
+  // Display the Simplex_tree
+  std::clog << "The complex contains " << st.num_simplices() << " simplices" << std::endl;
+  BOOST_CHECK(st.num_simplices() == 27);
+
+  bool simplex_is_changed = st.prune_above_dimension(200);
+  std::clog << "The complex pruned at 'dimension=3' contains " << st.num_simplices() << " simplices - dimension is"
+            << st.dimension() << std::endl;
+  BOOST_CHECK(!simplex_is_changed);
+  BOOST_CHECK(st.num_simplices() == 27);
+  // First check upper_bound_dimension(), then dimension() that recomputes it by a deep search
+  BOOST_CHECK(st.upper_bound_dimension() == 3);
+  BOOST_CHECK(st.dimension() == 3);
+
+  simplex_is_changed = st.prune_above_dimension(3);
+  std::clog << "The complex pruned at 'dimension=3' contains " << st.num_simplices() << " simplices - dimension is"
+            << st.dimension() << std::endl;
+  BOOST_CHECK(!simplex_is_changed);
+  BOOST_CHECK(st.num_simplices() == 27);
+  BOOST_CHECK(st.upper_bound_dimension() == 3);
+  BOOST_CHECK(st.dimension() == 3);
+
+  simplex_is_changed = st.prune_above_dimension(2);
+  std::clog << "The complex pruned at 'dimension=2' contains " << st.num_simplices() << " simplices - dimension is"
+            << st.dimension() << std::endl;
+  BOOST_CHECK(simplex_is_changed);
+  BOOST_CHECK(st.num_simplices() == 26);
+  BOOST_CHECK(st.upper_bound_dimension() == 2);
+  BOOST_CHECK(st.dimension() == 2);
+
+  simplex_is_changed = st.prune_above_dimension(1);
+  std::clog << "The complex pruned at 'dimension=1' contains " << st.num_simplices() << " simplices - dimension is"
+            << st.dimension() << std::endl;
+  BOOST_CHECK(simplex_is_changed);
+  BOOST_CHECK(st.num_simplices() == 20);
+  BOOST_CHECK(st.upper_bound_dimension() == 1);
+  BOOST_CHECK(st.dimension() == 1);
+
+  // Keep only the vertices
+  simplex_is_changed = st.prune_above_dimension(0);
+  std::clog << "The complex pruned at 'dimension=0' contains " << st.num_simplices() << " simplices - dimension is"
+            << st.dimension() << std::endl;
+  BOOST_CHECK(simplex_is_changed);
+  BOOST_CHECK(st.num_simplices() == 8);
+  BOOST_CHECK(st.upper_bound_dimension() == 0);
+  BOOST_CHECK(st.dimension() == 0);
+
+  // Remove all simplices, even vertices shall be possible
+  simplex_is_changed = st.prune_above_dimension(-1);
+  std::clog << "The complex pruned at 'dimension=-1' contains " << st.num_simplices() << " simplices - dimension is"
+            << st.dimension() << std::endl;
+  BOOST_CHECK(simplex_is_changed);
+  BOOST_CHECK(st.num_simplices() == 0);
+  BOOST_CHECK(st.upper_bound_dimension() == -1);
+  BOOST_CHECK(st.dimension() == -1);
+
+  // Test at the limit
+  simplex_is_changed = st.prune_above_dimension(-200);
+  std::clog << "The complex pruned at 'dimension=-1' contains " << st.num_simplices() << " simplices - dimension is"
+            << st.dimension() << std::endl;
+  BOOST_CHECK(!simplex_is_changed);
+  BOOST_CHECK(st.num_simplices() == 0);
+  BOOST_CHECK(st.upper_bound_dimension() == -1);
+  BOOST_CHECK(st.dimension() == -1);
+}
\ No newline at end of file
