@@ -31,8 +31,22 @@
 
 using namespace Gudhi;
 
-typedef boost::mpl::list<Simplex_tree<>, Simplex_tree<Simplex_tree_options_fast_persistence>> list_of_tested_variants;
+struct Simplex_tree_options_stable_simplex_handles {
+  typedef linear_indexing_tag Indexing_tag;
+  typedef int Vertex_handle;
+  typedef double Filtration_value;
+  typedef std::uint32_t Simplex_key;
+  static const bool store_key = true;
+  static const bool store_filtration = true;
+  static const bool contiguous_vertices = false;
+  static const bool link_nodes_by_label = true;
+  static const bool stable_simplex_handles = true;
+};
 
+typedef boost::mpl::list<Simplex_tree<>,
+                         Simplex_tree<Simplex_tree_options_fast_persistence>,
+                         Simplex_tree<Simplex_tree_options_full_featured>,
+                         Simplex_tree<Simplex_tree_options_stable_simplex_handles> > list_of_tested_variants;
 
 template<class typeST>
 void test_empty_simplex_tree(typeST& tst) {
@@ -41,6 +55,8 @@ void test_empty_simplex_tree(typeST& tst) {
   BOOST_CHECK(tst.null_vertex() == DEFAULT_VERTEX_VALUE);
   BOOST_CHECK(tst.num_vertices() == (size_t) 0);
   BOOST_CHECK(tst.num_simplices() == (size_t) 0);
+  BOOST_CHECK(tst.is_empty());
+  BOOST_CHECK(tst.num_simplices_by_dimension() == std::vector<size_t>());
   typename typeST::Siblings* STRoot = tst.root();
   BOOST_CHECK(STRoot != nullptr);
   BOOST_CHECK(STRoot->oncles() == nullptr);
@@ -86,7 +102,7 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(simplex_tree_when_empty, typeST, list_of_tested_va
   typeVectorVertex simplexVectorEmpty;
   BOOST_CHECK(simplexVectorEmpty.empty() == true);
   typePairSimplexBool returnEmptyValue = st.insert_simplex(simplexVectorEmpty, 0.0);
-  BOOST_CHECK(returnEmptyValue.first == typename typeST::Simplex_handle(nullptr));
+  BOOST_CHECK(returnEmptyValue.first == typeST::null_simplex());
   BOOST_CHECK(returnEmptyValue.second == true);
 
   test_empty_simplex_tree(st);
@@ -115,6 +131,7 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(simplex_tree_from_file, typeST, list_of_tested_var
   // Check
   BOOST_CHECK(st.num_simplices() == 143353);
   BOOST_CHECK(st.dimension() == 3);
+  BOOST_CHECK(!st.is_empty());
 
   int previous_size = 0;
   for (auto f_simplex : st.filtration_simplex_range()) {
@@ -154,7 +171,7 @@ void test_simplex_tree_insert_returns_true(const typePairSimplexBool& returnValu
   BOOST_CHECK(returnValue.second == true);
   // Simplex_handle = boost::container::flat_map< typeST::Vertex_handle, Node >::iterator
   typename typeST::Simplex_handle shReturned = returnValue.first;
-  BOOST_CHECK(shReturned != typename typeST::Simplex_handle(nullptr));
+  BOOST_CHECK(shReturned != typeST::null_simplex());
 }
 
 // Global variables
@@ -309,7 +326,7 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(simplex_tree_insertion, typeST, list_of_tested_var
   BOOST_CHECK(returnValue.second == false);
   // Simplex_handle = boost::container::flat_map< typeST::Vertex_handle, Node >::iterator
   typename typeST::Simplex_handle shReturned = returnValue.first;
-  BOOST_CHECK(shReturned == typename typeST::Simplex_handle(nullptr));
+  BOOST_CHECK(shReturned == typeST::null_simplex());
   std::clog << "st.num_vertices()=" << st.num_vertices() << std::endl;
   BOOST_CHECK(st.num_vertices() == (size_t) 4); // Not incremented !!
   BOOST_CHECK(st.dimension() == dim_max);
@@ -324,9 +341,10 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(simplex_tree_insertion, typeST, list_of_tested_var
   BOOST_CHECK(returnValue.second == false);
   // Simplex_handle = boost::container::flat_map< typeST::Vertex_handle, Node >::iterator
   shReturned = returnValue.first;
-  BOOST_CHECK(shReturned == typename typeST::Simplex_handle(nullptr));
+  BOOST_CHECK(shReturned == typeST::null_simplex());
   BOOST_CHECK(st.num_vertices() == (size_t) 4); // Not incremented !!
   BOOST_CHECK(st.dimension() == dim_max);
+  BOOST_CHECK(st.num_simplices_by_dimension() == std::vector<size_t>({4, 4, 1}));
 
   /* Inserted simplex:        */
   /*    1                     */
@@ -589,19 +607,31 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(NSimplexAndSubfaces_tree_insertion, typeST, list_o
 
 template<class typeST, class Vertex_handle>
 void test_cofaces(typeST& st, const std::vector<Vertex_handle>& expected, int dim, const std::vector<typename typeST::Simplex_handle>& res) {
-  typename typeST::Cofaces_simplex_range cofaces;
-  if (dim == 0)
-    cofaces = st.star_simplex_range(st.find(expected));
-  else
-    cofaces = st.cofaces_simplex_range(st.find(expected), dim);
-  for (auto simplex = cofaces.begin(); simplex != cofaces.end(); ++simplex) {
-    typename typeST::Simplex_vertex_range rg = st.simplex_vertex_range(*simplex);
-    for (auto vertex = rg.begin(); vertex != rg.end(); ++vertex) {
-      std::clog << "(" << *vertex << ")";
+  std::size_t nb_res = 0;
+  if (dim == 0) {
+    typename typeST::Cofaces_simplex_range stars = st.star_simplex_range(st.find(expected));
+    for (auto simplex = stars.begin(); simplex != stars.end(); ++simplex) {
+      typename typeST::Simplex_vertex_range rg = st.simplex_vertex_range(*simplex);
+      for (auto vertex = rg.begin(); vertex != rg.end(); ++vertex) {
+        std::clog << "(" << *vertex << ")";
+      }
+      std::clog << std::endl;
+      BOOST_CHECK(std::find(res.begin(), res.end(), *simplex) != res.end());
+      nb_res++;
+    }
+  } else {
+    typename typeST::Cofaces_simplex_range cofaces = st.cofaces_simplex_range(st.find(expected), dim);
+    for (auto simplex = cofaces.begin(); simplex != cofaces.end(); ++simplex) {
+      typename typeST::Simplex_vertex_range rg = st.simplex_vertex_range(*simplex);
+      for (auto vertex = rg.begin(); vertex != rg.end(); ++vertex) {
+        std::clog << "(" << *vertex << ")";
+      }
+      std::clog << std::endl;
+      BOOST_CHECK(std::find(res.begin(), res.end(), *simplex) != res.end());
+      nb_res++;
     }
-    std::clog << std::endl;
-    BOOST_CHECK(std::find(res.begin(), res.end(), *simplex) != res.end());
   }
+  BOOST_CHECK(nb_res == res.size());
 }
 
 BOOST_AUTO_TEST_CASE_TEMPLATE(coface_on_simplex_tree, typeST, list_of_tested_variants) {
@@ -622,11 +652,14 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(coface_on_simplex_tree, typeST, list_of_tested_var
   SimplexVector = {0, 1, 6, 7};
   st.insert_simplex_and_subfaces(SimplexVector);
 
+  SimplexVector = {4, 8};
+  st.insert_simplex_and_subfaces(SimplexVector);
+
   /* Inserted simplex:        */
   /*    1   6                 */
   /*    o---o                 */
-  /*   /X\7/                  */
-  /*  o---o---o---o           */
+  /*   /X\7/          8       */
+  /*  o---o---o---o---o       */
   /*  2   0   3\X/4           */
   /*            o             */
   /*            5             */
@@ -688,15 +721,84 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(coface_on_simplex_tree, typeST, list_of_tested_var
 
   std::clog << "Cofaces with a codimension too high (codimension + vetices > tree.dimension) :" << std::endl;
   test_cofaces(st, vertex, 5, result);
+  vertex.clear();
+  result.clear();
 
-  //std::clog << "Cofaces with an empty codimension" << std::endl;
-  //test_cofaces(st, vertex, -1, result);
-  //    std::clog << "Cofaces in an empty simplex tree" << std::endl;
-  //   typeST empty_tree;
-  //    test_cofaces(empty_tree, vertex, 1, result);
-  //std::clog << "Cofaces of an empty simplex" << std::endl;
-  //vertex.clear();
-  // test_cofaces(st, vertex, 1, result);
+  std::clog << "Star of (4):" << std::endl;
+
+  simplex_result = {4};
+  result.push_back(st.find(simplex_result));
+
+  simplex_result = {8, 4};
+  result.push_back(st.find(simplex_result));
+
+  simplex_result = {4, 3};
+  result.push_back(st.find(simplex_result));
+
+  simplex_result = {5, 4, 3};
+  result.push_back(st.find(simplex_result));
+
+  simplex_result = {5, 4};
+  result.push_back(st.find(simplex_result));
+  simplex_result.clear();
+
+  vertex = {4};
+  test_cofaces(st, vertex, 0, result);
+  vertex.clear();
+  result.clear();
+
+  std::clog << "Cofaces - codimension = 1 - of (4):" << std::endl;
+
+  simplex_result = {8, 4};
+  result.push_back(st.find(simplex_result));
+
+  simplex_result = {4, 3};
+  result.push_back(st.find(simplex_result));
+
+  simplex_result = {5, 4};
+  result.push_back(st.find(simplex_result));
+  simplex_result.clear();
+
+  vertex = {4};
+  test_cofaces(st, vertex, 1, result);
+  vertex.clear();
+  result.clear();
+
+  st.remove_maximal_simplex(st.find({4, 8}));
+  st.remove_maximal_simplex(st.find({8}));
+  std::clog << "Star of (4) after removal of simplices (8) and (4,8):" << std::endl;
+
+  simplex_result = {4};
+  result.push_back(st.find(simplex_result));
+
+  simplex_result = {4, 3};
+  result.push_back(st.find(simplex_result));
+
+  simplex_result = {5, 4, 3};
+  result.push_back(st.find(simplex_result));
+
+  simplex_result = {5, 4};
+  result.push_back(st.find(simplex_result));
+  simplex_result.clear();
+
+  vertex = {4};
+  test_cofaces(st, vertex, 0, result);
+  vertex.clear();
+  result.clear();
+
+  std::clog << "Cofaces - codimension = 1 - of (4) after removal of simplices (8) and (4,8):" << std::endl;
+
+  simplex_result = {4, 3};
+  result.push_back(st.find(simplex_result));
+
+  simplex_result = {5, 4};
+  result.push_back(st.find(simplex_result));
+  simplex_result.clear();
+
+  vertex = {4};
+  test_cofaces(st, vertex, 1, result);
+  vertex.clear();
+  result.clear();
 
 }
 
@@ -831,6 +933,10 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(simplex_tree_insert_graph, Graph, list_of_graph_va
   st1.insert_graph(g);
   BOOST_CHECK(st1.num_simplices() == 6);
 
+  Simplex_tree<Simplex_tree_options_stable_simplex_handles> sst1;
+  sst1.insert_graph(g);
+  BOOST_CHECK(sst1.num_simplices() == 6);
+
   // edges can have multiplicity in the graph unless we replace the first vecS with (hash_)setS
   add_edge(1, 0, 1.1, g);
   add_edge(1, 2, 3.3, g);
@@ -842,13 +948,24 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(simplex_tree_insert_graph, Graph, list_of_graph_va
   st2.insert_graph(g);
   BOOST_CHECK(st2.num_simplices() == 6);
 
+  Simplex_tree<Simplex_tree_options_stable_simplex_handles> sst2;
+  sst2.insert_graph(g);
+  BOOST_CHECK(sst2.num_simplices() == 6);
+
   std::clog << "st1 is" << std::endl;
   std::clog << st1 << std::endl;
 
   std::clog << "st2 is" << std::endl;
   std::clog << st2 << std::endl;
 
+  std::clog << "sst1 is" << std::endl;
+  std::clog << sst1 << std::endl;
+
+  std::clog << "sst2 is" << std::endl;
+  std::clog << sst2 << std::endl;
+
   BOOST_CHECK(st1 == st2);
+  BOOST_CHECK(sst1 == sst2);
 }
 
 BOOST_AUTO_TEST_CASE_TEMPLATE(insert_duplicated_vertices, typeST, list_of_tested_variants) {
@@ -1039,8 +1156,12 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(simplex_tree_boundaries_and_opposite_vertex_iterat
   }
 }
 
-BOOST_AUTO_TEST_CASE(batch_vertices) {
-  typedef Simplex_tree<> typeST;
+typedef boost::mpl::list<Simplex_tree<>,
+                         Simplex_tree<Simplex_tree_options_full_featured>,
+                         Simplex_tree<Simplex_tree_options_stable_simplex_handles> >
+                           list_of_tested_variants_wo_fast_persistence;
+
+BOOST_AUTO_TEST_CASE_TEMPLATE(batch_vertices, typeST, list_of_tested_variants_wo_fast_persistence) {
   std::clog << "********************************************************************" << std::endl;
   std::clog << "TEST BATCH VERTEX INSERTION" << std::endl;
   typeST st;
@@ -1051,4 +1172,72 @@ BOOST_AUTO_TEST_CASE(batch_vertices) {
   BOOST_CHECK(st.num_simplices() == 4);
   BOOST_CHECK(st.filtration(st.find({2})) == 0.);
   BOOST_CHECK(st.filtration(st.find({3})) == 1.5);
+
+  for (auto coface : st.star_simplex_range(st.find({5}))) {
+    std::clog << "coface";
+    for (auto vertex : st.simplex_vertex_range(coface)) {
+      std::clog << " " << vertex;
+      // Should be the only one
+      BOOST_CHECK(vertex == 5);
+    }
+    std::clog << "\n";
+  }
+}
+
+BOOST_AUTO_TEST_CASE_TEMPLATE(simplex_tree_clear, typeST, list_of_tested_variants) {
+  std::clog << "********************************************************************" << std::endl;
+  std::clog << "TEST SIMPLEX TREE CLEAR" << std::endl;
+  typeST st;
+  st.insert_simplex_and_subfaces({0, 1}, 1.5);
+  st.initialize_filtration();
+  st.clear();
+  BOOST_CHECK(st.num_vertices() == 0);
+  BOOST_CHECK(st.num_simplices() == 0);
+  BOOST_CHECK(st.upper_bound_dimension() == -1);
+  BOOST_CHECK(st.dimension() == -1);
+  BOOST_CHECK(boost::size(st.filtration_simplex_range()) == 0);
+  typeST st_empty;
+  BOOST_CHECK(st == st_empty);
+  st.insert_simplex_and_subfaces({0}, 2.5);
+  BOOST_CHECK(boost::size(st.cofaces_simplex_range(st.find({0}), 1)) == 0);
+}
+
+BOOST_AUTO_TEST_CASE_TEMPLATE(for_each_simplex_skip_iteration, typeST, list_of_tested_variants) {
+  std::clog << "********************************************************************" << std::endl;
+  std::clog << "TEST FOR_EACH ITERATION SKIP MECHANISM" << std::endl;
+  typeST st;
+
+  st.insert_simplex_and_subfaces({2, 1, 0}, 3.);
+  st.insert_simplex_and_subfaces({3, 0}, 2.);
+  st.insert_simplex_and_subfaces({3, 4, 5}, 3.);
+  st.insert_simplex_and_subfaces({0, 1, 6, 7}, 4.);
+
+  /* Inserted simplex:        */
+  /*    1   6                 */
+  /*    o---o                 */
+  /*   /X\7/                  */
+  /*  o---o---o---o           */
+  /*  2   0   3\X/4           */
+  /*            o             */
+  /*            5             */
+
+  std::vector<size_t> num_simplices_by_dim_until_two(2);
+  auto lambda_nb_simp_by_dim = [&num_simplices_by_dim_until_two](typename typeST::Simplex_handle, int dim)
+                                  {
+                                    BOOST_CHECK (dim < 2);
+                                    ++num_simplices_by_dim_until_two[dim];
+                                    return dim >= 1; // The iteration will skip the children in this case
+                                  };
+  st.for_each_simplex(lambda_nb_simp_by_dim);
+  for (auto num_simplices : num_simplices_by_dim_until_two)
+    std::cout << num_simplices << ", ";
+  std::cout << std::endl;
+
+  auto num_simplices_by_dim = st.num_simplices_by_dimension();
+  for (auto num_simplices : num_simplices_by_dim)
+    std::cout << num_simplices << ", ";
+  std::cout << std::endl;
+
+  BOOST_CHECK(num_simplices_by_dim_until_two[0] == num_simplices_by_dim[0]);
+  BOOST_CHECK(num_simplices_by_dim_until_two[1] == num_simplices_by_dim[1]);
 }
