@@ -10,6 +10,7 @@
 #   - 2021/11 Vincent Rouvreau: factorize _automatic_sample_range
 
 import numpy as np
+from scipy.spatial.distance import cdist
 from sklearn.base          import BaseEstimator, TransformerMixin
 from sklearn.exceptions    import NotFittedError
 from sklearn.preprocessing import MinMaxScaler, MaxAbsScaler
@@ -21,7 +22,7 @@ except ImportError:
     # Will be removed in 1.3
     from sklearn.neighbors     import DistanceMetric
 
-from .preprocessing import DiagramScaler, BirthPersistenceTransform
+from .preprocessing import DiagramScaler, BirthPersistenceTransform, _maybe_fit_transform
 
 #############################################
 # Finite Vectorization methods ##############
@@ -53,14 +54,15 @@ class PersistenceImage(BaseEstimator, TransformerMixin):
             y (n x 1 array): persistence diagram labels (unused).
         """
         if np.isnan(np.array(self.im_range)).any():
-            try:
+            if all(len(d) == 0 for d in X):
+                self.im_range_fixed_ = self.im_range
+            else:
                 new_X = BirthPersistenceTransform().fit_transform(X)
                 pre = DiagramScaler(use=True, scalers=[([0], MinMaxScaler()), ([1], MinMaxScaler())]).fit(new_X,y)
                 [mx,my],[Mx,My] = [pre.scalers[0][1].data_min_[0], pre.scalers[1][1].data_min_[0]], [pre.scalers[0][1].data_max_[0], pre.scalers[1][1].data_max_[0]]
-                self.im_range = np.where(np.isnan(np.array(self.im_range)), np.array([mx, Mx, my, My]), np.array(self.im_range))
-            except ValueError:
-                # Empty persistence diagram case - https://github.com/GUDHI/gudhi-devel/issues/507
-                pass
+                self.im_range_fixed_ = np.where(np.isnan(np.array(self.im_range)), np.array([mx, Mx, my, My]), np.array(self.im_range))
+        else:
+            self.im_range_fixed_ = self.im_range
         return self
 
     def transform(self, X):
@@ -84,11 +86,11 @@ class PersistenceImage(BaseEstimator, TransformerMixin):
             for j in range(num_pts_in_diag):
                 w[j] = self.weight(diagram[j,:])
 
-            x_values, y_values = np.linspace(self.im_range[0], self.im_range[1], self.resolution[0]), np.linspace(self.im_range[2], self.im_range[3], self.resolution[1])
+            x_values, y_values = np.linspace(self.im_range_fixed_[0], self.im_range_fixed_[1], self.resolution[0]), np.linspace(self.im_range_fixed_[2], self.im_range_fixed_[3], self.resolution[1])
             Xs, Ys = np.tile((diagram[:,0][:,np.newaxis,np.newaxis]-x_values[np.newaxis,np.newaxis,:]),[1,self.resolution[1],1]), np.tile(diagram[:,1][:,np.newaxis,np.newaxis]-y_values[np.newaxis,:,np.newaxis],[1,1,self.resolution[0]])
             image = np.tensordot(w, np.exp((-np.square(Xs)-np.square(Ys))/(2*np.square(self.bandwidth)))/(np.square(self.bandwidth)*2*np.pi), 1)
 
-            Xfit.append(image.flatten()[np.newaxis,:])
+            Xfit.append(image.reshape(1,-1))
 
         Xfit = np.concatenate(Xfit, 0)
 
@@ -97,6 +99,7 @@ class PersistenceImage(BaseEstimator, TransformerMixin):
     def __call__(self, diag):
         """
         Apply PersistenceImage on a single persistence diagram and outputs the result.
+        If :func:`fit` hasn't been run, this uses `fit_transform` on a clone of the object and thus does not affect later calls.
 
         Parameters:
             diag (n x 2 numpy array): input persistence diagram.
@@ -104,7 +107,7 @@ class PersistenceImage(BaseEstimator, TransformerMixin):
         Returns:
             numpy array with shape (number of pixels = **resolution[0]** x **resolution[1]**):: output persistence image.
         """
-        return self.fit_transform([diag])[0,:]
+        return _maybe_fit_transform(self, 'im_range_fixed_', diag)
 
 def _automatic_sample_range(sample_range, X):
         """
@@ -139,14 +142,14 @@ def _trim_endpoints(x, are_endpoints_nan):
 
 def _grid_from_sample_range(self, X):
     sample_range = np.array(self.sample_range)
-    self.nan_in_range = np.isnan(sample_range)
-    self.new_resolution = self.resolution
+    self.nan_in_range_ = np.isnan(sample_range)
+    self.new_resolution_ = self.resolution
     if not self.keep_endpoints:
-        self.new_resolution += self.nan_in_range.sum()
-    self.sample_range_fixed = _automatic_sample_range(sample_range, X)
-    self.grid_ = np.linspace(self.sample_range_fixed[0], self.sample_range_fixed[1], self.new_resolution)
+        self.new_resolution_ += self.nan_in_range_.sum()
+    self.sample_range_fixed_ = _automatic_sample_range(sample_range, X)
+    self.grid_ = np.linspace(self.sample_range_fixed_[0], self.sample_range_fixed_[1], self.new_resolution_)
     if not self.keep_endpoints:
-        self.grid_ = _trim_endpoints(self.grid_, self.nan_in_range)
+        self.grid_ = _trim_endpoints(self.grid_, self.nan_in_range_)
 
 
 class Landscape(BaseEstimator, TransformerMixin):
@@ -214,6 +217,7 @@ class Landscape(BaseEstimator, TransformerMixin):
     def __call__(self, diag):
         """
         Apply Landscape on a single persistence diagram and outputs the result.
+        If :func:`fit` hasn't been run, this uses `fit_transform` on a clone of the object and thus does not affect later calls.
 
         Parameters:
             diag (n x 2 numpy array): input persistence diagram.
@@ -221,7 +225,7 @@ class Landscape(BaseEstimator, TransformerMixin):
         Returns:
             numpy array with shape (number of samples = **num_landscapes** x **resolution**): output persistence landscape.
         """
-        return self.fit_transform([diag])[0, :]
+        return _maybe_fit_transform(self, 'grid_', diag)
 
 class Silhouette(BaseEstimator, TransformerMixin):
     """
@@ -281,6 +285,7 @@ class Silhouette(BaseEstimator, TransformerMixin):
     def __call__(self, diag):
         """
         Apply Silhouette on a single persistence diagram and outputs the result.
+        If :func:`fit` hasn't been run, this uses `fit_transform` on a clone of the object and thus does not affect later calls.
 
         Parameters:
             diag (n x 2 numpy array): input persistence diagram.
@@ -288,7 +293,7 @@ class Silhouette(BaseEstimator, TransformerMixin):
         Returns:
             numpy array with shape (**resolution**): output persistence silhouette.
         """
-        return self.fit_transform([diag])[0,:]
+        return _maybe_fit_transform(self, 'grid_', diag)
 
 
 class BettiCurve(BaseEstimator, TransformerMixin):
@@ -351,7 +356,7 @@ class BettiCurve(BaseEstimator, TransformerMixin):
 
         if self.predefined_grid is None:
             if self.resolution is None: # Flexible/exact version
-                events = np.unique(np.concatenate([pd.flatten() for pd in X] + [[-np.inf]], axis=0))
+                events = np.unique(np.concatenate([pd.ravel() for pd in X] + [[-np.inf]], axis=0))
                 self.grid_ = np.array(events)
             else:
                 _grid_from_sample_range(self, X)
@@ -379,7 +384,7 @@ class BettiCurve(BaseEstimator, TransformerMixin):
         
         N = len(X)
 
-        events = np.concatenate([pd.flatten(order="F") for pd in X], axis=0)
+        events = np.concatenate([pd.ravel(order="F") for pd in X], axis=0)
         sorting = np.argsort(events)
         offsets = np.zeros(1 + N, dtype=int)
         for i in range(0, N):
@@ -412,7 +417,7 @@ class BettiCurve(BaseEstimator, TransformerMixin):
 
             N = len(X)
 
-            events = np.concatenate([pd.flatten(order="F") for pd in X], axis=0)
+            events = np.concatenate([pd.ravel(order="F") for pd in X], axis=0)
             sorting = np.argsort(events)
             offsets = np.zeros(1 + N, dtype=int)
             for i in range(0, N):
@@ -445,8 +450,9 @@ class BettiCurve(BaseEstimator, TransformerMixin):
     def __call__(self, diag):
         """
         Shorthand for transform on a single persistence diagram.
+        If :func:`fit` hasn't been run, this uses `fit_transform` on a clone of the object and thus does not affect later calls.
         """
-        return self.fit_transform([diag])[0, :]
+        return _maybe_fit_transform(self, 'grid_', diag)
 
 
 
@@ -509,8 +515,8 @@ class Entropy(BaseEstimator, TransformerMixin):
                 ent = np.zeros(self.resolution)
                 for j in range(num_pts_in_diag):
                     [px,py] = orig_diagram[j,:2]
-                    min_idx = np.clip(np.ceil((px - self.sample_range_fixed[0]) / self.step_).astype(int), 0, self.resolution)
-                    max_idx = np.clip(np.ceil((py - self.sample_range_fixed[0]) / self.step_).astype(int), 0, self.resolution)
+                    min_idx = np.clip(np.ceil((px - self.sample_range_fixed_[0]) / self.step_).astype(int), 0, self.resolution)
+                    max_idx = np.clip(np.ceil((py - self.sample_range_fixed_[0]) / self.step_).astype(int), 0, self.resolution)
                     ent[min_idx:max_idx]-=p[j]*np.log(p[j])
                 if self.normalized:
                     ent = ent / np.linalg.norm(ent, ord=1)
@@ -522,6 +528,7 @@ class Entropy(BaseEstimator, TransformerMixin):
     def __call__(self, diag):
         """
         Apply Entropy on a single persistence diagram and outputs the result.
+        If :func:`fit` hasn't been run, this uses `fit_transform` on a clone of the object and thus does not affect later calls.
 
         Parameters:
             diag (n x 2 numpy array): input persistence diagram.
@@ -529,7 +536,7 @@ class Entropy(BaseEstimator, TransformerMixin):
         Returns:
             numpy array with shape (1 if **mode** = "scalar" else **resolution**): output entropy.
         """
-        return self.fit_transform([diag])[0,:]
+        return _maybe_fit_transform(self, 'grid_', diag)
 
 class TopologicalVector(BaseEstimator, TransformerMixin):
     """
@@ -600,7 +607,7 @@ class TopologicalVector(BaseEstimator, TransformerMixin):
         Returns:
             numpy array with shape (**threshold**): output topological vector.
         """
-        return self.fit_transform([diag])[0,:]
+        return self.transform([diag])[0,:]
 
 class ComplexPolynomial(BaseEstimator, TransformerMixin):
     """
@@ -672,19 +679,22 @@ class ComplexPolynomial(BaseEstimator, TransformerMixin):
         Returns:
             numpy array with shape (**threshold**): output complex vector of coefficients.
         """
-        return self.fit_transform([diag])[0,:]
+        return self.transform([diag])[0,:]
 
 def _lapl_contrast(measure, centers, inertias):
-    """contrast function for vectorising `measure` in ATOL"""
-    return np.exp(-pairwise.pairwise_distances(measure, Y=centers) / inertias)
+    """contrast function for vectorising `measure` in ATOL
+    we use cdist so as to accept 'inf' values instead of raising ValueError with sklearn.pairwise"""
+    return np.exp(-cdist(XA=measure, XB=centers) / inertias)
 
 def _gaus_contrast(measure, centers, inertias):
-    """contrast function for vectorising `measure` in ATOL"""
-    return np.exp(-pairwise.pairwise_distances(measure, Y=centers, squared=True) / inertias**2)
+    """contrast function for vectorising `measure` in ATOL
+    we use cdist so as to accept 'inf' values instead of raising ValueError with sklearn.pairwise"""
+    return np.exp(-cdist(XA=measure, XB=centers, metric="sqeuclidean") / inertias**2)
 
 def _indicator_contrast(diags, centers, inertias):
-    """contrast function for vectorising `measure` in ATOL"""
-    robe_curve = np.clip(2-pairwise.pairwise_distances(diags, Y=centers)/inertias, 0, 1)
+    """contrast function for vectorising `measure` in ATOL
+    we use cdist so as to accept 'inf' values instead of raising ValueError with sklearn.pairwise"""
+    robe_curve = np.clip(2-cdist(XA=diags, XB=centers)/inertias, 0, 1)
     return robe_curve
 
 def _cloud_weighting(measure):
@@ -739,15 +749,21 @@ class Atol(BaseEstimator, TransformerMixin):
                 (default: gaussian contrast function, see page 3 in the ATOL paper).
         """
         self.quantiser = quantiser
-        self.contrast = {
+        self.contrast = contrast
+        self.weighting_method = weighting_method
+
+    def get_contrast(self):
+        return {
             "gaussian": _gaus_contrast,
             "laplacian": _lapl_contrast,
             "indicator": _indicator_contrast,
-        }.get(contrast, _gaus_contrast)
-        self.weighting_method = {
+        }.get(self.contrast, _gaus_contrast)
+
+    def get_weighting_method(self):
+        return {
             "cloud"   : _cloud_weighting,
             "iidproba": _iidproba_weighting,
-        }.get(weighting_method, _cloud_weighting)
+        }.get(self.weighting_method, _cloud_weighting)
 
     def fit(self, X, y=None, sample_weight=None):
         """
@@ -765,18 +781,24 @@ class Atol(BaseEstimator, TransformerMixin):
         """
         if not hasattr(self.quantiser, 'fit'):
             raise TypeError("quantiser %s has no `fit` attribute." % (self.quantiser))
+
+        # In fitting we remove infinite death time points so that every center is finite
+        X = [dgm[~np.isinf(dgm).any(axis=1), :] for dgm in X]
+
         if sample_weight is None:
-            sample_weight = np.concatenate([self.weighting_method(measure) for measure in X])
+            sample_weight = [self.get_weighting_method()(measure) for measure in X]
 
         measures_concat = np.concatenate(X)
-        self.quantiser.fit(X=measures_concat, sample_weight=sample_weight)
+        weights_concat = np.concatenate(sample_weight)
+        self.quantiser.fit(X=measures_concat, sample_weight=weights_concat)
         self.centers = self.quantiser.cluster_centers_
         # Hack, but some people are unhappy if the order depends on the version of sklearn
         self.centers = self.centers[np.lexsort(self.centers.T)]
         if self.quantiser.n_clusters == 1:
             dist_centers = pairwise.pairwise_distances(measures_concat)
             np.fill_diagonal(dist_centers, 0)
-            self.inertias = np.array([np.max(dist_centers)/2])
+            best_inertia = np.max(dist_centers)/2 if np.max(dist_centers)/2 > 0 else 1
+            self.inertias = np.array([best_inertia])
         else:
             dist_centers = pairwise.pairwise_distances(self.centers)
             dist_centers[dist_centers == 0] = np.inf
@@ -785,7 +807,7 @@ class Atol(BaseEstimator, TransformerMixin):
 
     def __call__(self, measure, sample_weight=None):
         """
-        Apply measure vectorisation on a single measure.
+        Apply measure vectorisation on a single measure. Only available after `fit` has been called.
 
         Parameters:
             measure (n x d numpy array): input measure in R^d.
@@ -794,8 +816,8 @@ class Atol(BaseEstimator, TransformerMixin):
             numpy array in R^self.quantiser.n_clusters.
         """
         if sample_weight is None:
-            sample_weight = self.weighting_method(measure)
-        return np.sum(sample_weight * self.contrast(measure, self.centers, self.inertias.T).T, axis=1)
+            sample_weight = self.get_weighting_method()(measure)
+        return np.sum(sample_weight * self.get_contrast()(measure, self.centers, self.inertias.T).T, axis=1)
 
     def transform(self, X, sample_weight=None):
         """
@@ -811,5 +833,5 @@ class Atol(BaseEstimator, TransformerMixin):
             numpy array with shape (number of measures) x (self.quantiser.n_clusters).
         """
         if sample_weight is None:
-            sample_weight = [self.weighting_method(measure) for measure in X]
+            sample_weight = [self.get_weighting_method()(measure) for measure in X]
         return np.stack([self(measure, sample_weight=weight) for measure, weight in zip(X, sample_weight)])
